import java.awt.*;

/**
 * Piece colors enum
 */
public enum PieceColor {

    /**
     * Black color with white border
     */
    BLACK(Color.BLACK, Color.WHITE),

    /**
     * White color with black border
     */
    WHITE(Color.WHITE, Color.BLACK);

    /**
     * Fill color
     */
    Color fill;

    /**
     * Draw color
     */
    Color draw;

    /**
     * PieceColor constructor
     * @param fillColor fill color
     * @param drawColor draw color
     */
    PieceColor(Color fillColor, Color drawColor) {
        setColors(fillColor, drawColor);
    }

    /**
     * Set piece's fill and draw colors
     * @param fillColor piece's fill color
     * @param drawColor piece's border color
     */
    public void setColors(Color fillColor, Color drawColor) {
        fill = fillColor;
        draw = drawColor;
    }

    /**
     * Set theme to the piece color
     * @param theme new theme
     */
    public static void setTheme(Theme theme) {
        PieceColor.WHITE.setColors(theme.whiteFill, theme.whiteBorder);
        PieceColor.BLACK.setColors(theme.blackFill, theme.blackBorder);
    }

}
